class User < ApplicationRecord
  has_secure_password
  has_many :sessions, dependent: :destroy
  has_many :images, dependent: :destroy

  normalizes :email_address, with: ->(e) { e.strip.downcase }

  validates :email_address, presence: true, uniqueness: true
  validates :name, presence: true

  # Session security settings
  attribute :require_two_factor, :boolean, default: false
  attribute :session_timeout_minutes, :integer, default: 60
  attribute :notify_on_new_login, :boolean, default: true
  attribute :max_sessions, :integer, default: 5

  # Admin role
  # enum role: { user: 0, admin: 1 }, _default: :user

  def admin?
    role == "admin"
  end
  
  def security_settings
    {
      require_two_factor: require_two_factor,
      session_timeout_minutes: session_timeout_minutes,
      notify_on_new_login: notify_on_new_login,
      max_sessions: max_sessions
    }
  end
  
  def enforce_max_sessions
    return unless max_sessions > 0
    
    # Get all sessions except the current one, ordered by last activity
    other_sessions = sessions.where.not(id: Current.session&.id).order(updated_at: :desc)
    
    # If we have more sessions than allowed, destroy the oldest ones
    if other_sessions.count >= max_sessions
      sessions_to_remove = other_sessions.offset(max_sessions - 1)
      sessions_to_remove.destroy_all
    end
  end
  
  def notify_on_new_login?
    notify_on_new_login
  end
end
